@@ -10,9 +10,11 @@ import yaml
 from astropy.time import TimeDelta
 from glob import glob
 from pyoperators import (
-    I, BlockRowOperator, DenseBlockDiagonalOperator, HomothetyOperator,
-    IdentityOperator, ReshapeOperator, Rotation2dOperator, Rotation3dOperator)
+    BlockRowOperator, DenseBlockDiagonalOperator, HomothetyOperator,
+    IdentityOperator, ReshapeOperator, Rotation2dOperator, Rotation3dOperator,
+    I, MPI, rule_manager)
 from pyoperators.utils import ifirst
+from pyoperators.utils.mpi import as_mpi
 from pysimulators import (
     Acquisition, CartesianEquatorial2HorizontalOperator,
     CartesianGalactic2EquatorialOperator,
@@ -22,48 +24,95 @@ from pysimulators import (
 from pysimulators.interfaces.healpy import HealpixConvolutionGaussianOperator
 from .calibration import QubicCalibration
 from .instrument import QubicInstrument
+from .scene import QubicScene
 
 __all__ = ['QubicAcquisition']
 
 
 class QubicAcquisition(Acquisition):
     """
-    The QubicAcquisition class, which represents the instrument and
-    pointing setups.
+    The QubicAcquisition class, which combines the instrument, sampling and
+    scene models.
 
     """
-    def __init__(self, instrument, pointing, block_id=None, selection=None):
+    def __init__(self, instrument, sampling, scene=None, calibration=None,
+                 detector_sigma=10, detector_fknee=0, detector_fslope=1,
+                 detector_ncorr=10, detector_tau=0.01, ngrids=None,
+                 synthbeam_fraction=0.99, kind='IQU', nside=256,
+                 nprocs_instrument=None, nprocs_sampling=None, comm=None):
         """
+        acq = QubicAcquisition(instrument, sampling, [scene=|kind=, nside=],
+                               nprocs_instrument=, nprocs_sampling=, comm=)
+        acq = QubicAcquisition(band, sampling, [scene=|kind=, nside=],
+                               nprocs_instrument=, nprocs_sampling=, comm=)
+
         Parameters
         ----------
-        instrument : str or QubicInstrument
+        band : int
+            The sky frequency, in GHz.
+        scene : QubicScene, optional
+            The discretized observed scene (the sky).
+        kind : 'I', 'QU', 'IQU', optional
+            The sky kind: 'I' for intensity-only, 'QU' for Q and U maps,
+            and 'IQU' for intensity plus QU maps.
+        nside : int, optional
+            The Healpix scene's nside.
+        instrument : QubicInstrument, optional
             Module name (only 'monochromatic' for now), or a QubicInstrument
             instance.
-        pointing : array-like of shape (n,3) or sequence of
-            The triplets (θ,φ,ψ), where (φ,θ,ψ) are the Euler angles
-            of the intrinsic ZY'Z'' rotations. Note the ordering of the angles.
-            θ : co-latitude
-            φ : longitude
-            ψ : minus the position angle
-        block_id : string or sequence of, optional
-           The pointing block identifier.
-        selection : integer or sequence of, optional
-           The indices of the pointing sequence to be selected to construct
-           the pointing configuration.
+        calibration : QubicCalibration, optional
+            The calibration tree.
+        detector_tau : array-like, optional
+            The detector time constants in seconds.
+        detector_sigma : array-like, optional
+            The standard deviation of the detector white noise component.
+        detector_fknee : array-like, optional
+            The detector 1/f knee frequency in Hertz.
+        detector_fslope : array-like, optional
+            The detector 1/f slope index.
+        detector_ncorr : int, optional
+            The detector 1/f correlation length.
+        synthbeam_fraction: float, optional
+            The fraction of significant peaks retained for the computation
+            of the synthetic beam.
+        ngrids : int, optional
+            Number of detector grids.
+        nprocs_instrument : int
+            For a given sampling slice, number of procs dedicated to
+            the instrument.
+        nprocs_sampling : int
+            For a given detector slice, number of procs dedicated to
+            the sampling.
+        comm : mpi4py.MPI.Comm
+            The acquisition's MPI communicator. Note that it is transformed
+            into a 2d cartesian communicator before being stored as the 'comm'
+            attribute. The following relationship must hold:
+                comm.size = nprocs_instrument * nprocs_sampling
 
         """
-        if not isinstance(instrument, (QubicInstrument, str)):
-            raise TypeError("Invalid type for the instrument ('{}' instead of "
-                            "'QubicInstrument' or 'str').".format(type(
-                                instrument).__name__))
-        if isinstance(instrument, str):
-            raise NotImplementedError('Module names not fixed yet.')
-        Acquisition.__init__(self, instrument, pointing, block_id=block_id,
-                             selection=selection)
-        # XXX HACK
-        from .pointings import QubicPointing
-        if isinstance(pointing, QubicPointing):
-            self.pointing = pointing
+        if scene is None:
+            if isinstance(instrument, QubicInstrument):
+                try:
+                    scene = instrument._deprecated_sky
+                except AttributeError:
+                    scene = 150
+            else:
+                scene = instrument
+        if not isinstance(scene, QubicScene):
+            scene = QubicScene(scene, kind=kind, nside=nside)
+
+        if not isinstance(instrument, QubicInstrument):
+            if ngrids is None:
+                ngrids = 1 if scene.kind == 'I' else 2
+            instrument = QubicInstrument(
+                calibration=calibration, detector_sigma=detector_sigma,
+                detector_fknee=detector_fknee, detector_fslope=detector_fslope,
+                detector_ncorr=detector_ncorr, detector_tau=detector_tau,
+                synthbeam_fraction=synthbeam_fraction, ngrids=ngrids)
+
+        Acquisition.__init__(self, instrument, sampling, scene,
+                             nprocs_instrument=nprocs_instrument,
+                             nprocs_sampling=nprocs_sampling, comm=comm)
 
     def get_hitmap(self, nside=None):
         """
@@ -72,23 +121,26 @@ class QubicAcquisition(Acquisition):
 
         """
         if nside is None:
-            nside = self.instrument.sky.nside
-        ipixel = self.pointing.tohealpix(nside)
+            nside = self.scene.nside
+        ipixel = self.sampling.healpix(nside)
         npixel = 12 * nside**2
-        return np.histogram(ipixel, bins=npixel, range=(0, npixel))[0]
+        hit = np.histogram(ipixel, bins=npixel, range=(0, npixel))[0]
+        self.sampling.comm.Allreduce(MPI.IN_PLACE, as_mpi(hit), op=MPI.SUM)
+        return hit
 
     def get_rotation(self):
         """
         Return the instrument-to-galactic rotation matrix.
 
         """
-        p = self.pointing
+        p = self.sampling
         time = p.date_obs + TimeDelta(p.time, format='sec')
-        r = CartesianEquatorial2GalacticOperator() * \
-            CartesianHorizontal2EquatorialOperator(
-                'NE', time, p.latitude, p.longitude) * \
-            Rotation3dOperator("ZY'Z''", p.azimuth, 90 - p.elevation, p.pitch,
-                               degrees=True)
+        with rule_manager(none=False):
+            r = CartesianEquatorial2GalacticOperator() * \
+                CartesianHorizontal2EquatorialOperator(
+                    'NE', time, p.latitude, p.longitude) * \
+                Rotation3dOperator("ZY'Z''", p.azimuth, 90 - p.elevation,
+                                   p.pitch, degrees=True)
         return r
 
     def get_rotation_g2i(self):
@@ -96,13 +148,14 @@ class QubicAcquisition(Acquisition):
         Return the galactic-to-instrument rotation matrix.
 
         """
-        p = self.pointing
+        p = self.sampling
         time = p.date_obs + TimeDelta(p.time, format='sec')
-        r = Rotation3dOperator("ZY'Z''", p.azimuth, 90 - p.elevation, p.pitch,
-                               degrees=True).T * \
-            CartesianEquatorial2HorizontalOperator(
-                'NE', time, p.latitude, p.longitude) * \
-            CartesianGalactic2EquatorialOperator()
+        with rule_manager(none=False):
+            r = Rotation3dOperator("ZY'Z''", p.azimuth, 90 - p.elevation,
+                                   p.pitch, degrees=True).T * \
+                CartesianEquatorial2HorizontalOperator(
+                    'NE', time, p.latitude, p.longitude) * \
+                CartesianGalactic2EquatorialOperator()
         return r
 
     def get_convolution_peak_operator(self, fwhm=np.radians(0.64883707),
@@ -126,9 +179,9 @@ class QubicAcquisition(Acquisition):
 
         """
         if tau is None:
-            tau = self.instrument.detector.packed.tau
-        sampling_period = self.pointing.get_sampling_period()
-        shapein = (self.get_ndetectors(), self.pointing.size)
+            tau = self.instrument.detector.tau
+        sampling_period = self.sampling.period
+        shapein = (len(self.instrument), len(self.sampling))
         if sampling_period == 0:
             return IdentityOperator(shapein)
         return ConvolutionTruncatedExponentialOperator(tau / sampling_period,
@@ -139,22 +192,22 @@ class QubicAcquisition(Acquisition):
         Return the rotation matrix for the half-wave plate.
 
         """
-        if self.instrument.sky.kind == 'I':
-            return IdentityOperator()
-        if self.instrument.sky.kind == 'QU':
-            return Rotation2dOperator(-4 * self.pointing.angle_hwp,
-                                      degrees=True)
-        return Rotation3dOperator('X', -4 * self.pointing.angle_hwp,
-                                  degrees=True)
+        shape = (len(self.instrument), len(self.sampling))
+        if self.scene.kind == 'I':
+            return IdentityOperator(shapein=shape)
+        if self.scene.kind == 'QU':
+            return Rotation2dOperator(-4 * self.sampling.angle_hwp,
+                                      degrees=True, shapein=shape + (2,))
+        return Rotation3dOperator('X', -4 * self.sampling.angle_hwp,
+                                  degrees=True, shapein=shape + (3,))
 
     def get_invntt_operator(self):
         """
         Return the inverse time-time noise correlation matrix as an Operator.
 
         """
-
-        l = self.instrument.detector.packed
-        fs = 1 / self.pointing.get_sampling_period()
+        l = self.instrument.detector
+        fs = 1 / self.sampling.period
         return Acquisition.get_invntt_operator(
             self, sigma=l.sigma, fknee=l.fknee, fslope=l.fslope,
             sampling_frequency=fs, ncorr=self.instrument.detector.ncorr)
@@ -164,18 +217,34 @@ class QubicAcquisition(Acquisition):
         Return a noisy timeline.
 
         """
-        l = self.instrument.detector.packed
-        fs = 1 / self.pointing.get_sampling_period()
+        l = self.instrument.detector
+        fs = 1 / self.sampling.period
         return Acquisition.get_noise(
             self, sigma=l.sigma, fknee=l.fknee, fslope=l.fslope,
             sampling_frequency=fs, out=out)
 
+    def get_operator(self):
+        """
+        Return the operator of the acquisition.
+
+        """
+        projection = self.get_projection_operator()
+        hwp = self.get_hwp_operator()
+        polarizer = self.get_polarizer_operator()
+        response = self.get_detector_response_operator()
+
+        with rule_manager(inplace=True):
+            H = response * polarizer * (hwp * projection)
+        if self.scene == 'QU':
+            H = self.get_subtract_grid_operator() * H
+        return H
+
     def get_polarizer_operator(self):
         """
         Return operator for the polarizer grid.
 
         """
-        if self.instrument.sky.kind == 'I':
+        if self.scene.kind == 'I':
             return HomothetyOperator(1 / self.instrument.detector.ngrids)
 
         if self.instrument.detector.ngrids == 1:
@@ -183,9 +252,9 @@ class QubicAcquisition(Acquisition):
                 'Polarized input not handled by a single detector grid.')
 
         nd = len(self.instrument)
-        nt = len(self.pointing)
-        grid = self.instrument.detector.packed.quadrant // 4
-        z = np.zeros(self.get_ndetectors())
+        nt = len(self.sampling)
+        grid = self.instrument.detector.quadrant // 4
+        z = np.zeros(nd)
         data = np.array([z + 0.5, 0.5 - grid, z]).T[:, None, None, :]
         return ReshapeOperator((nd, nt, 1), (nd, nt)) * \
             DenseBlockDiagonalOperator(data)
@@ -206,14 +275,14 @@ class QubicAcquisition(Acquisition):
         partitionin = 2 * (len(self.instrument) // 2,)
         return BlockRowOperator([I, -I], axisin=0, partitionin=partitionin)
 
-    def get_projection_peak_operator(self, rotation=None, dtype=None,
-                                     synthbeam_fraction=None, verbose=True):
+    def get_projection_operator(self, rotation=None, dtype=None,
+                                synthbeam_fraction=None, verbose=True):
         """
         Return the projection operator for the peak sampling.
 
         Parameters
         ----------
-        rotation : array (npointings, 3, 3)
+        rotation : array (nsamplings, 3, 3)
             The Instrument-to-Reference rotation matrix.
         dtype : dtype
             The datatype of the elements in the projection matrix.
@@ -223,9 +292,9 @@ class QubicAcquisition(Acquisition):
         """
         if rotation is None:
             rotation = self.get_rotation_g2i()
-        return self.instrument.get_projection_peak_operator(
-            rotation, synthbeam_fraction=synthbeam_fraction, dtype=dtype,
-            verbose=verbose)
+        return self.instrument.get_projection_operator(
+            self.scene, rotation, synthbeam_fraction=synthbeam_fraction,
+            dtype=dtype, verbose=verbose)
 
     @classmethod
     def load(cls, filename, instrument=None, selection=None):
