@@ -9,14 +9,16 @@ import numexpr as ne
 import numpy as np
 from pyoperators import Spherical2CartesianOperator, MPI
 from pyoperators.utils import strenum
-from pysimulators import Instrument, Layout, ProjectionOperator
+from pysimulators import Instrument, LayoutVertex, ProjectionOperator
 from pysimulators.interfaces.healpy import Cartesian2HealpixOperator
 from pysimulators.sparse import (
     FSRMatrix, FSRRotation2dMatrix, FSRRotation3dMatrix)
 from scipy.constants import c, pi
 from . import _flib as flib
 from .calibration import QubicCalibration
-from .utils import _compress_mask, _uncompress_mask
+from .scene import QubicScene
+from .utils import _compress_mask
+from .warnings import QubicDeprecationWarning, warn
 
 __all__ = ['QubicInstrument']
 
@@ -26,21 +28,17 @@ class QubicInstrument(Instrument):
     The QubicInstrument class. It represents the instrument setup.
 
     """
-    def __init__(self, name, calibration=None, removed=None,
+    def __init__(self, name=None, calibration=None,
                  detector_sigma=10, detector_fknee=0, detector_fslope=1,
                  detector_ncorr=10, detector_tau=0.01,
-                 synthbeam_fraction=0.99, ngrids=None, nside=256,
-                 commin=MPI.COMM_WORLD, commout=MPI.COMM_WORLD, **keywords):
+                 synthbeam_fraction=0.99, ngrids=2, nside=256, **keywords):
         """
         Parameters
         ----------
         name : str
-            The module name. So far, only 'monochromatic,nopol' and
-            'monochromatic' are available.
+            Deprecated.
         calibration : QubicCalibration
             The calibration tree.
-        removed : str or 2D-array of bool
-            Array specifying which bolometers are removed.
         detector_tau : array-like
             The detector time constants in seconds.
         detector_sigma : array-like
@@ -52,13 +50,12 @@ class QubicInstrument(Instrument):
         detector_ncorr : int
             The detector 1/f correlation length.
         nside : int, optional
-            The Healpix nside of the sky.
+            Deprecated.
         synthbeam_fraction: float, optional
             The fraction of significant peaks retained for the computation
             of the synthetic beam.
         ngrids : int, optional
-            Number of detector grids. (Default: 2 for the polarized case,
-            1 otherwise)
+            Number of detector grids.
         nu : float, optional
             The operating monochromatic frequency or the filter central
             frequency, in Hz.
@@ -66,30 +63,34 @@ class QubicInstrument(Instrument):
             The filter width.
 
         """
+        if name is not None:
+            skind = ", kind='I'" if 'nopol' in name else ''
+            snside = ', nside={0}'.format(nside) if nside is not None else ''
+            warn('Please update your code:\nacq = QubicAcquisition(150, sam'
+                 'pling{0}{1})\n'.format(skind, snside),
+                 QubicDeprecationWarning)
+            name = name.replace(' ', '').lower()
+            names = 'monochromatic', 'monochromatic,qu', 'monochromatic,nopol'
+            if name not in names:
+                raise ValueError(
+                    "The only modes implemented are {0}.".format(
+                        strenum(names, 'and')))
+            self._init_deprecated_sky(name, nside)
+
         if calibration is None:
             calibration = QubicCalibration()
-        name = name.replace(' ', '').lower()
-        names = 'monochromatic', 'monochromatic,qu', 'monochromatic,nopol'
-        if name not in names:
-            raise ValueError(
-                "The only modes implemented are {0}.".format(
-                    strenum(names, 'and')))
         self.calibration = calibration
         layout = self._get_detector_layout(
-            name, removed, ngrids, detector_sigma, detector_fknee,
-            detector_fslope, detector_ncorr, detector_tau)
-        Instrument.__init__(self, name, layout, commin=commin, commout=commout)
-        self._init_sky(nside)
+            ngrids, detector_sigma, detector_fknee, detector_fslope,
+            detector_ncorr, detector_tau)
+        Instrument.__init__(self, 'QUBIC', layout)
         self._init_primary_beam()
         self._init_optics(**keywords)
         self._init_horns()
         self._init_synthetic_beam(synthbeam_fraction)
 
-    def _get_detector_layout(self, name, removed, ngrids, sigma, fknee, fslope,
-                             ncorr, tau):
-        polarized = 'nopol' not in name.split(',')
-        if ngrids is None:
-            ngrids = 2 if polarized else 1
+    def _get_detector_layout(self, ngrids, sigma, fknee, fslope, ncorr,
+                             tau):
         shape, vertex, removed_, index, quadrant = \
             self.calibration.get('detarray')
         if ngrids == 2:
@@ -98,31 +99,21 @@ class QubicInstrument(Instrument):
             removed_ = np.array([removed_, removed_])
             index = np.array([index, index + np.max(index) + 1], index.dtype)
             quadrant = np.array([quadrant, quadrant + 4], quadrant.dtype)
-        if removed is not None:
-            if isinstance(removed, str):
-                removed = _uncompress_mask(removed).reshape(shape)
-            removed_ |= removed
-        layout = Layout(shape, vertex=vertex, removed=removed_, index=index,
-                        quadrant=quadrant, sigma=sigma, fknee=fknee,
-                        fslope=fslope, tau=tau)
+        layout = LayoutVertex(
+            shape, 4, vertex=vertex, selection=~removed_, ordering=index,
+            quadrant=quadrant, sigma=sigma, fknee=fknee, fslope=fslope,
+            tau=tau)
         layout.ncorr = ncorr
         layout.ngrids = ngrids
         return layout
 
-    def _init_sky(self, nside):
-        class Sky(object):
-            pass
-        names = self.name.split(',')
-        size = 12 * nside**2
-        self.sky = Sky()
-        self.sky.size = size
-        self.sky.shape = (size,) if 'nopol' in names \
-                                 else (size, 2) if 'qu' in names \
-                                                else (size, 3)
-        self.sky.nside = nside
-        self.sky.kind = 'I' if 'nopol' in names \
-                            else 'QU' if 'qu' in names \
-                                      else 'IQU'
+    def _init_deprecated_sky(self, name, nside):
+        names = name.split(',')
+        kind = 'I' if 'nopol' in names \
+                   else 'QU' if 'qu' in names \
+                             else 'IQU'
+        sky = QubicScene(150, kind=kind, nside=nside)
+        self._deprecated_sky = sky
 
     def _init_primary_beam(self):
         class PrimaryBeam(object):
@@ -134,13 +125,11 @@ class QubicInstrument(Instrument):
                 return np.exp(-theta**2 / (2 * self.sigma**2))
         self.primary_beam = PrimaryBeam(self.calibration.get('primbeam'))
 
-    def _init_optics(self, nu=150e9, dnu_nu=0, **keywords):
+    def _init_optics(self, **keywords):
         class Optics(object):
             pass
         optics = Optics()
         optics.focal_length = self.calibration.get('optics')['focal length']
-        optics.nu = nu
-        optics.dnu_nu = dnu_nu
         self.optics = optics
 
     def _init_horns(self):
@@ -154,13 +143,9 @@ class QubicInstrument(Instrument):
         self.synthetic_beam = sb
 
     def __str__(self):
-        state = [('name', self.name),
-                 ('nu', self.optics.nu),
-                 ('dnu_nu', self.optics.dnu_nu),
-                 ('nside', self.sky.nside),
-                 ('synthbeam_fraction', self.synthetic_beam.fraction),
+        state = [('synthbeam_fraction', self.synthetic_beam.fraction),
                  ('ngrids', self.detector.ngrids),
-                 ('removed', _compress_mask(self.detector.removed))]
+                 ('selection', _compress_mask(~self.detector.all.removed))]
         return 'Instrument:\n' + \
                '\n'.join(['    ' + a + ': ' + repr(v) for a, v in state]) + \
                '\n\nCalibration:\n' + '\n'. \
@@ -174,15 +159,15 @@ class QubicInstrument(Instrument):
 
         """
         a = mp.gca()
-        vertex = self.detector.packed.vertex.reshape((-1, 4, 2))
+        vertex = self.detector.vertex
         for v in vertex:
             a.add_patch(mp.Polygon(v, closed=True, fill=False, **keywords))
         if autoscale:
             mp.autoscale()
         mp.show()
 
-    def get_projection_peak_operator(self, rotation, dtype=None,
-                                     synthbeam_fraction=None, verbose=True):
+    def get_projection_operator(self, scene, rotation, dtype=None,
+                                synthbeam_fraction=None, verbose=True):
         """
         Return the peak sampling operator.
 
@@ -190,6 +175,8 @@ class QubicInstrument(Instrument):
         ----------
         rotation : ndarray (ntimes, 3, 3)
             The Reference-to-Instrument rotation matrix.
+        scene : QubicScene
+            The observed scene.
         dtype : dtype
             The datatype of the elements in the projection matrix.
         synthbeam_fraction : float, optional
@@ -201,14 +188,15 @@ class QubicInstrument(Instrument):
         if dtype is None:
             dtype = np.float32
         dtype = np.dtype(dtype)
-        ndetectors = len(self.detector.packed)
+        ndetectors = len(self)
         if rotation.data.ndim == 2:
             ntimes = 1
         else:
             ntimes = rotation.data.shape[0]
-        nside = self.sky.nside
+        nside = scene.nside
 
-        theta, phi, vals = _peak_angles_fraction(self, synthbeam_fraction)
+        theta, phi, vals = _peak_angles_fraction(self, scene,
+                                                 synthbeam_fraction)
         ncolmax = theta.shape[-1]
         thetaphi = _pack_vector(theta, phi)  # (ndetectors, ncolmax, 2)
         direction = Spherical2CartesianOperator('zenith,azimuth')(thetaphi)
@@ -220,8 +208,8 @@ class QubicInstrument(Instrument):
 
         cls = {'I': FSRMatrix,
                'QU': FSRRotation2dMatrix,
-               'IQU': FSRRotation3dMatrix}[self.sky.kind]
-        ndims = len(self.sky.kind)
+               'IQU': FSRRotation3dMatrix}[scene.kind]
+        ndims = len(scene.kind)
         s = cls((ndetectors * ntimes * ndims, 12 * nside**2 * ndims),
                 ncolmax=ncolmax, dtype=dtype, dtype_index=dtype_index,
                 verbose=verbose)
@@ -233,7 +221,7 @@ class QubicInstrument(Instrument):
             e_ni = rotation.T(e_nf[i].swapaxes(0, 1)).swapaxes(0, 1)
             index[i] = Cartesian2HealpixOperator(nside)(e_ni)
 
-        if self.sky.kind == 'I':
+        if scene.kind == 'I':
             value = s.data.value.reshape(ndetectors, ntimes, ncolmax)
             value[...] = vals[:, None, :]
             shapeout = (ndetectors, ntimes)
@@ -248,7 +236,7 @@ class QubicInstrument(Instrument):
                 raise TypeError(
                     'The projection matrix cannot be created with types: {0} a'
                     'nd {1}.'.format(dtype, dtype_index))
-            if self.sky.kind == 'QU':
+            if scene.kind == 'QU':
                 shapeout = (ndetectors, ntimes, 2)
             else:
                 shapeout = (ndetectors, ntimes, 3)
@@ -256,7 +244,7 @@ class QubicInstrument(Instrument):
         return ProjectionOperator(s, shapeout=shapeout)
 
 
-def _peak_angles(q, kmax):
+def _peak_angles(q, scene, kmax):
     """
     Return the spherical coordinates (theta,phi) of the beam peaks, in radians.
 
@@ -268,9 +256,9 @@ def _peak_angles(q, kmax):
         (2 * kmax + 1)**2 = 25 peaks and a value of kmax=0 will only sample
         the central peak.
     """
-    ndetector = len(q.detector.packed)
-    center = q.detector.packed.center
-    lmbda = c / q.optics.nu
+    ndetector = len(q)
+    center = q.detector.center
+    lmbda = c / scene.nu
     dx = q.horn.spacing
     detvec = np.vstack([-center[..., 0],
                         -center[..., 1],
@@ -286,10 +274,10 @@ def _peak_angles(q, kmax):
     return theta, phi
 
 
-def _peak_angles_fraction(q, fraction):
+def _peak_angles_fraction(q, scene, fraction):
 
     # there is no need to go beyond kmax=5
-    theta, phi = _peak_angles(q, kmax=5)
+    theta, phi = _peak_angles(q, scene, kmax=5)
     index = _argsort(theta)
     theta = theta[index]
     phi = phi[index]
